# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 14.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V38
4980
4981
12346
12347
12874
12875
12890
12891
21317
21318
65810
65811
65893
65894
66284
66285
68043
68044
69227
69228
69412
69413
69725
69726
70124
70125
71916
71917
72805
72806
119659
119660
126076
126077
126220
126221
133507
133508
END
